@@ -8,7 +8,7 @@ import (
 	"testing"
 
 	"github.com/aws/aws-sdk-go-v2/aws"
-	awsmiddleware "github.com/aws/aws-sdk-go-v2/aws/middleware"
+	"github.com/aws/aws-sdk-go-v2/internal/awstesting"
 	"github.com/aws/aws-sdk-go-v2/internal/awstesting/unit"
 	"github.com/aws/aws-sdk-go-v2/service/s3control"
 	"github.com/aws/smithy-go/middleware"
@@ -96,8 +96,8 @@ func TestUpdateEndpointBuild(t *testing.T) {
 								func(options *s3control.Options) {
 									options.APIOptions = append(options.APIOptions,
 										func(stack *middleware.Stack) error {
-											stack.Serialize.Insert(&fm,
-												"OperationSerializer", middleware.Before)
+											stack.Finalize.Insert(&fm,
+												"Signing", middleware.Before)
 											return nil
 										})
 
@@ -134,6 +134,7 @@ func TestEndpointWithARN(t *testing.T) {
 	cases := map[string]struct {
 		options                    s3control.Options
 		bucket                     string
+		accessPoint                string
 		expectedErr                string
 		expectedReqURL             string
 		expectedSigningName        string
@@ -142,82 +143,82 @@ func TestEndpointWithARN(t *testing.T) {
 		expectedHeaderForAccountID bool
 	}{
 		"Outpost AccessPoint with no S3UseARNRegion flag set": {
-			bucket: "arn:aws:s3-outposts:us-west-2:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint",
+			accessPoint: "arn:aws:s3-outposts:us-west-2:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint",
 			options: s3control.Options{
 				Region: "us-west-2",
 			},
-			expectedReqURL:             "https://s3-outposts.us-west-2.amazonaws.com/v20180820/bucket/myaccesspoint",
+			expectedReqURL:             "https://s3-outposts.us-west-2.amazonaws.com/v20180820/accesspoint/myaccesspoint",
 			expectedSigningName:        "s3-outposts",
 			expectedSigningRegion:      "us-west-2",
 			expectedHeaderForAccountID: true,
 			expectedHeaderForOutpostID: "op-01234567890123456",
 		},
 		"Outpost AccessPoint Cross-Region Enabled": {
-			bucket: "arn:aws:s3-outposts:us-east-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint",
+			accessPoint: "arn:aws:s3-outposts:us-east-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint",
 			options: s3control.Options{
 				Region:       "us-west-2",
 				UseARNRegion: true,
 			},
-			expectedReqURL:             "https://s3-outposts.us-east-1.amazonaws.com/v20180820/bucket/myaccesspoint",
+			expectedReqURL:             "https://s3-outposts.us-east-1.amazonaws.com/v20180820/accesspoint/myaccesspoint",
 			expectedSigningName:        "s3-outposts",
 			expectedSigningRegion:      "us-east-1",
 			expectedHeaderForAccountID: true,
 			expectedHeaderForOutpostID: "op-01234567890123456",
 		},
 		"Outpost AccessPoint Cross-Region Disabled": {
-			bucket: "arn:aws:s3-outposts:us-east-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint",
+			accessPoint: "arn:aws:s3-outposts:us-east-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint",
 			options: s3control.Options{
 				Region: "us-west-2",
 			},
-			expectedErr: "client region does not match provided ARN region",
+			expectedErr: "Invalid configuration: region from ARN `us-east-1` does not match client region `us-west-2` and UseArnRegion is `false`",
 		},
 		"Outpost AccessPoint other partition": {
-			bucket: "arn:aws-cn:s3-outposts:cn-north-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint",
+			accessPoint: "arn:aws-cn:s3-outposts:cn-north-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint",
 			options: s3control.Options{
 				Region:       "us-west-2",
 				UseARNRegion: true,
 			},
-			expectedErr: "ConfigurationError : client partition does not match provided ARN partition",
+			expectedErr: "Client was configured for partition `aws` but ARN has `aws-cn`",
 		},
 		"Outpost AccessPoint us-gov region": {
-			bucket: "arn:aws-us-gov:s3-outposts:us-gov-east-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint",
+			accessPoint: "arn:aws-us-gov:s3-outposts:us-gov-east-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint",
 			options: s3control.Options{
 				Region:       "us-gov-east-1",
 				UseARNRegion: true,
 			},
-			expectedReqURL:             "https://s3-outposts.us-gov-east-1.amazonaws.com/v20180820/bucket/myaccesspoint",
+			expectedReqURL:             "https://s3-outposts.us-gov-east-1.amazonaws.com/v20180820/accesspoint/myaccesspoint",
 			expectedSigningName:        "s3-outposts",
 			expectedSigningRegion:      "us-gov-east-1",
 			expectedHeaderForAccountID: true,
 			expectedHeaderForOutpostID: "op-01234567890123456",
 		},
 		"Outpost AccessPoint with client region as FIPS": {
-			bucket: "arn:aws-us-gov:s3-outposts:us-gov-east-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint",
+			accessPoint: "arn:aws-us-gov:s3-outposts:us-gov-east-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint",
 			options: s3control.Options{
 				Region: "us-gov-east-1",
 				EndpointOptions: s3control.EndpointResolverOptions{
 					UseFIPSEndpoint: aws.FIPSEndpointStateEnabled,
 				},
 			},
-			expectedReqURL:             "https://s3-outposts-fips.us-gov-east-1.amazonaws.com/v20180820/bucket/myaccesspoint",
+			expectedReqURL:             "https://s3-outposts-fips.us-gov-east-1.amazonaws.com/v20180820/accesspoint/myaccesspoint",
 			expectedSigningName:        "s3-outposts",
 			expectedSigningRegion:      "us-gov-east-1",
 			expectedHeaderForOutpostID: "op-01234567890123456",
 			expectedHeaderForAccountID: true,
 		},
 		"Outpost AccessPoint with client region as FIPS (ResolvedRegion)": {
-			bucket: "arn:aws-us-gov:s3-outposts:us-gov-east-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint",
+			accessPoint: "arn:aws-us-gov:s3-outposts:us-gov-east-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint",
 			options: s3control.Options{
 				Region: "us-gov-east-1-fips",
 			},
-			expectedReqURL:             "https://s3-outposts-fips.us-gov-east-1.amazonaws.com/v20180820/bucket/myaccesspoint",
+			expectedReqURL:             "https://s3-outposts-fips.us-gov-east-1.amazonaws.com/v20180820/accesspoint/myaccesspoint",
 			expectedSigningName:        "s3-outposts",
 			expectedSigningRegion:      "us-gov-east-1",
 			expectedHeaderForOutpostID: "op-01234567890123456",
 			expectedHeaderForAccountID: true,
 		},
 		"Outpost AccessPoint with client FIPS and use arn region enabled": {
-			bucket: "arn:aws-us-gov:s3-outposts:us-gov-east-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint",
+			accessPoint: "arn:aws-us-gov:s3-outposts:us-gov-east-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint",
 			options: s3control.Options{
 				Region: "us-gov-east-1",
 				EndpointOptions: s3control.EndpointResolverOptions{
@@ -225,26 +226,26 @@ func TestEndpointWithARN(t *testing.T) {
 				},
 				UseARNRegion: true,
 			},
-			expectedReqURL:             "https://s3-outposts-fips.us-gov-east-1.amazonaws.com/v20180820/bucket/myaccesspoint",
+			expectedReqURL:             "https://s3-outposts-fips.us-gov-east-1.amazonaws.com/v20180820/accesspoint/myaccesspoint",
 			expectedSigningName:        "s3-outposts",
 			expectedSigningRegion:      "us-gov-east-1",
 			expectedHeaderForOutpostID: "op-01234567890123456",
 			expectedHeaderForAccountID: true,
 		},
 		"Outpost AccessPoint with client FIPS (ResolvedRegion) and use arn region enabled": {
-			bucket: "arn:aws-us-gov:s3-outposts:us-gov-east-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint",
+			accessPoint: "arn:aws-us-gov:s3-outposts:us-gov-east-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint",
 			options: s3control.Options{
 				Region:       "us-gov-east-1-fips",
 				UseARNRegion: true,
 			},
-			expectedReqURL:             "https://s3-outposts-fips.us-gov-east-1.amazonaws.com/v20180820/bucket/myaccesspoint",
+			expectedReqURL:             "https://s3-outposts-fips.us-gov-east-1.amazonaws.com/v20180820/accesspoint/myaccesspoint",
 			expectedSigningName:        "s3-outposts",
 			expectedSigningRegion:      "us-gov-east-1",
 			expectedHeaderForOutpostID: "op-01234567890123456",
 			expectedHeaderForAccountID: true,
 		},
 		"Outpost AccessPoint client FIPS and cross region ARN": {
-			bucket: "arn:aws-us-gov:s3-outposts:us-gov-west-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint",
+			accessPoint: "arn:aws-us-gov:s3-outposts:us-gov-west-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint",
 			options: s3control.Options{
 				Region:       "us-gov-east-1",
 				UseARNRegion: true,
@@ -252,26 +253,26 @@ func TestEndpointWithARN(t *testing.T) {
 					UseFIPSEndpoint: aws.FIPSEndpointStateEnabled,
 				},
 			},
-			expectedReqURL:             "https://s3-outposts-fips.us-gov-west-1.amazonaws.com/v20180820/bucket/myaccesspoint",
+			expectedReqURL:             "https://s3-outposts-fips.us-gov-west-1.amazonaws.com/v20180820/accesspoint/myaccesspoint",
 			expectedSigningName:        "s3-outposts",
 			expectedSigningRegion:      "us-gov-west-1",
 			expectedHeaderForOutpostID: "op-01234567890123456",
 			expectedHeaderForAccountID: true,
 		},
 		"Outpost AccessPoint client FIPS (ResolvedRegion) and cross region ARN": {
-			bucket: "arn:aws-us-gov:s3-outposts:us-gov-west-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint",
+			accessPoint: "arn:aws-us-gov:s3-outposts:us-gov-west-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint",
 			options: s3control.Options{
 				Region:       "us-gov-east-1-fips",
 				UseARNRegion: true,
 			},
-			expectedReqURL:             "https://s3-outposts-fips.us-gov-west-1.amazonaws.com/v20180820/bucket/myaccesspoint",
+			expectedReqURL:             "https://s3-outposts-fips.us-gov-west-1.amazonaws.com/v20180820/accesspoint/myaccesspoint",
 			expectedSigningName:        "s3-outposts",
 			expectedSigningRegion:      "us-gov-west-1",
 			expectedHeaderForOutpostID: "op-01234567890123456",
 			expectedHeaderForAccountID: true,
 		},
 		"Outpost AccessPoint client FIPS with valid ARN region": {
-			bucket: "arn:aws-us-gov:s3-outposts:us-gov-east-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint",
+			accessPoint: "arn:aws-us-gov:s3-outposts:us-gov-east-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint",
 			options: s3control.Options{
 				Region: "us-gov-east-1",
 				EndpointOptions: s3control.EndpointResolverOptions{
@@ -279,70 +280,61 @@ func TestEndpointWithARN(t *testing.T) {
 				},
 				UseARNRegion: true,
 			},
-			expectedReqURL:             "https://s3-outposts-fips.us-gov-east-1.amazonaws.com/v20180820/bucket/myaccesspoint",
+			expectedReqURL:             "https://s3-outposts-fips.us-gov-east-1.amazonaws.com/v20180820/accesspoint/myaccesspoint",
 			expectedSigningName:        "s3-outposts",
 			expectedSigningRegion:      "us-gov-east-1",
 			expectedHeaderForOutpostID: "op-01234567890123456",
 			expectedHeaderForAccountID: true,
 		},
 		"Outpost AccessPoint client FIPS (ResolvedRegion) with valid ARN region": {
-			bucket: "arn:aws-us-gov:s3-outposts:us-gov-east-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint",
+			accessPoint: "arn:aws-us-gov:s3-outposts:us-gov-east-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint",
 			options: s3control.Options{
 				Region:       "us-gov-east-1-fips",
 				UseARNRegion: true,
 			},
-			expectedReqURL:             "https://s3-outposts-fips.us-gov-east-1.amazonaws.com/v20180820/bucket/myaccesspoint",
+			expectedReqURL:             "https://s3-outposts-fips.us-gov-east-1.amazonaws.com/v20180820/accesspoint/myaccesspoint",
 			expectedSigningName:        "s3-outposts",
 			expectedSigningRegion:      "us-gov-east-1",
 			expectedHeaderForOutpostID: "op-01234567890123456",
 			expectedHeaderForAccountID: true,
 		},
-		"Outpost AccessPoint with DualStack": {
-			bucket: "arn:aws:s3-outposts:us-west-2:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint",
-			options: s3control.Options{
-				Region:       "us-west-2",
-				UseARNRegion: true,
-				UseDualstack: true,
-			},
-			expectedErr: "ConfigurationError : client configured for S3 Dual-stack but is not supported with resource ARN",
-		},
 		"Invalid outpost resource format": {
 			bucket: "arn:aws:s3-outposts:us-west-2:123456789012:outpost",
 			options: s3control.Options{
 				Region: "us-west-2",
 			},
-			expectedErr: "outpost resource-id not set",
+			expectedErr: "operation error S3 Control: GetBucket, invalid Amazon s3-outposts ARN",
 		},
 		"Missing access point for outpost resource": {
 			bucket: "arn:aws:s3-outposts:us-west-2:123456789012:outpost:op-01234567890123456",
 			options: s3control.Options{
 				Region: "us-west-2",
 			},
-			expectedErr: "incomplete outpost resource type",
+			expectedErr: "operation error S3 Control: GetBucket, invalid Amazon s3-outposts ARN",
 		},
 		"access point": {
-			bucket: "myaccesspoint",
+			accessPoint: "myaccesspoint",
 			options: s3control.Options{
 				Region: "us-west-2",
 			},
-			expectedReqURL:             "https://123456789012.s3-control.us-west-2.amazonaws.com/v20180820/bucket/myaccesspoint",
+			expectedReqURL:             "https://123456789012.s3-control.us-west-2.amazonaws.com/v20180820/accesspoint/myaccesspoint",
 			expectedHeaderForAccountID: true,
 			expectedSigningRegion:      "us-west-2",
 			expectedSigningName:        "s3",
 		},
 		"outpost access point with unsupported sub-resource": {
-			bucket: "arn:aws:s3-outposts:us-west-2:123456789012:outpost:op-01234567890123456:accesspoint:mybucket:object:foo",
+			accessPoint: "arn:aws:s3-outposts:us-west-2:123456789012:outpost:op-01234567890123456:accesspoint:mybucket:object:foo",
 			options: s3control.Options{
 				Region: "us-west-2",
 			},
 			expectedErr: "sub resource not supported",
 		},
 		"Missing outpost identifiers in outpost access point arn": {
-			bucket: "arn:aws:s3-outposts:us-west-2:123456789012:accesspoint:myendpoint",
+			accessPoint: "arn:aws:s3-outposts:us-west-2:123456789012:accesspoint:myendpoint",
 			options: s3control.Options{
 				Region: "us-west-2",
 			},
-			expectedErr: "invalid Amazon s3-outposts ARN",
+			expectedErr: "operation error S3 Control: GetAccessPoint, invalid Amazon s3-outposts ARN",
 		},
 		"Outpost Bucket with no S3UseARNRegion flag set": {
 			bucket: "arn:aws:s3-outposts:us-west-2:123456789012:outpost:op-01234567890123456:bucket:mybucket",
@@ -372,7 +364,7 @@ func TestEndpointWithARN(t *testing.T) {
 			options: s3control.Options{
 				Region: "us-west-2",
 			},
-			expectedErr: "client region does not match provided ARN region",
+			expectedErr: "Invalid configuration: region from ARN `us-east-1` does not match client region `us-west-2` and UseArnRegion is `false`",
 		},
 		"Outpost Bucket other partition": {
 			bucket: "arn:aws-cn:s3-outposts:cn-north-1:123456789012:outpost:op-01234567890123456:bucket:mybucket",
@@ -380,7 +372,7 @@ func TestEndpointWithARN(t *testing.T) {
 				Region:       "us-west-2",
 				UseARNRegion: true,
 			},
-			expectedErr: "ConfigurationError : client partition does not match provided ARN partition",
+			expectedErr: "Client was configured for partition `aws` but ARN has `aws-cn`",
 		},
 		"Outpost Bucket us-gov region": {
 			bucket: "arn:aws-us-gov:s3-outposts:us-gov-east-1:123456789012:outpost:op-01234567890123456:bucket:mybucket",
@@ -402,14 +394,14 @@ func TestEndpointWithARN(t *testing.T) {
 					UseFIPSEndpoint: aws.FIPSEndpointStateEnabled,
 				},
 			},
-			expectedErr: "ConfigurationError : client region does not match provided ARN region",
+			expectedErr: "Invalid configuration: region from ARN `us-gov-east-1` does not match client region `us-gov-west-1` and UseArnRegion is `false`",
 		},
 		"Outpost Bucket client FIPS (ResolvedRegion), cross-region ARN": {
 			bucket: "arn:aws-us-gov:s3-outposts:us-gov-east-1:123456789012:outpost:op-01234567890123456:bucket:mybucket",
 			options: s3control.Options{
 				Region: "us-gov-west-1-fips",
 			},
-			expectedErr: "ConfigurationError : client region does not match provided ARN region",
+			expectedErr: "Invalid configuration: region from ARN `us-gov-east-1` does not match client region `us-gov-west-1` and UseArnRegion is `false`",
 		},
 		"Outpost Bucket client FIPS with non cross-region ARN region": {
 			bucket: "arn:aws-us-gov:s3-outposts:us-gov-east-1:123456789012:outpost:op-01234567890123456:bucket:mybucket",
@@ -438,27 +430,19 @@ func TestEndpointWithARN(t *testing.T) {
 			expectedHeaderForOutpostID: "op-01234567890123456",
 			expectedHeaderForAccountID: true,
 		},
-		"Outpost Bucket with DualStack": {
-			bucket: "arn:aws:s3-outposts:us-west-2:123456789012:outpost:op-01234567890123456:bucket:mybucket",
-			options: s3control.Options{
-				Region:       "us-west-2",
-				UseDualstack: true,
-			},
-			expectedErr: "ConfigurationError : client configured for S3 Dual-stack but is not supported with resource ARN",
-		},
 		"Missing bucket id": {
 			bucket: "arn:aws:s3-outposts:us-west-2:123456789012:outpost:op-01234567890123456:bucket",
 			options: s3control.Options{
 				Region: "us-west-2",
 			},
-			expectedErr: "invalid Amazon s3-outposts ARN",
+			expectedErr: "operation error S3 Control: GetBucket, invalid Amazon s3-outposts ARN",
 		},
 		"Invalid ARN": {
 			bucket: "arn:aws:s3-outposts:us-west-2:123456789012:bucket:mybucket",
 			options: s3control.Options{
 				Region: "us-west-2",
 			},
-			expectedErr: "invalid Amazon s3-outposts ARN, unknown resource type",
+			expectedErr: "operation error S3 Control: GetBucket, invalid Amazon s3-outposts ARN",
 		},
 		"Invalid Outpost Bucket ARN with FIPS pseudo-region (prefix)": {
 			bucket: "arn:aws:s3-outposts:fips-us-east-1:123456789012:outpost:op-01234567890123456:bucket:mybucket",
@@ -477,7 +461,7 @@ func TestEndpointWithARN(t *testing.T) {
 			expectedErr: "FIPS region not allowed in ARN",
 		},
 		"Invalid Outpost AccessPoint ARN with FIPS pseudo-region (prefix)": {
-			bucket: "arn:aws-us-gov:s3-outposts:fips-us-east-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint",
+			accessPoint: "arn:aws-us-gov:s3-outposts:fips-us-east-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint",
 			options: s3control.Options{
 				Region:       "us-west-2",
 				UseARNRegion: true,
@@ -485,7 +469,7 @@ func TestEndpointWithARN(t *testing.T) {
 			expectedErr: "FIPS region not allowed in ARN",
 		},
 		"Invalid Outpost AccessPoint ARN with FIPS pseudo-region (suffix)": {
-			bucket: "arn:aws-us-gov:s3-outposts:us-east-1-fips:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint",
+			accessPoint: "arn:aws-us-gov:s3-outposts:us-east-1-fips:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint",
 			options: s3control.Options{
 				Region:       "us-west-2",
 				UseARNRegion: true,
@@ -510,19 +494,18 @@ func TestEndpointWithARN(t *testing.T) {
 
 			ctx := context.Background()
 
-			// call an operation
-			_, err := svc.GetBucket(ctx, &s3control.GetBucketInput{
-				Bucket:    ptr.String(c.bucket),
-				AccountId: ptr.String("123456789012"),
-			}, func(options *s3control.Options) {
-				// append request retriever middleware for request inspection
-				options.APIOptions = append(options.APIOptions,
-					func(stack *middleware.Stack) error {
-						// adds AFTER operation serializer middleware
-						stack.Serialize.Insert(&fm, "OperationSerializer", middleware.After)
-						return nil
-					})
-			})
+			var err error
+			if len(c.accessPoint) > 0 {
+				_, err = svc.GetAccessPoint(ctx, &s3control.GetAccessPointInput{
+					Name:      ptr.String(c.accessPoint),
+					AccountId: ptr.String("123456789012"),
+				}, addRequestRetriever(&fm))
+			} else {
+				_, err = svc.GetBucket(ctx, &s3control.GetBucketInput{
+					Bucket:    ptr.String(c.bucket),
+					AccountId: ptr.String("123456789012"),
+				}, addRequestRetriever(&fm))
+			}
 
 			// inspect any errors
 			if len(c.expectedErr) != 0 {
@@ -583,15 +566,7 @@ func TestCustomEndpoint_SpecialOperations(t *testing.T) {
 				return svc.CreateBucket(ctx, &s3control.CreateBucketInput{
 					Bucket:    aws.String("mockBucket"),
 					OutpostId: aws.String("op-01234567890123456"),
-				}, func(options *s3control.Options) {
-					// append request retriever middleware for request inspection
-					options.APIOptions = append(options.APIOptions,
-						func(stack *middleware.Stack) error {
-							// adds AFTER operation serializer middleware
-							stack.Serialize.Insert(fm, "OperationSerializer", middleware.After)
-							return nil
-						})
-				})
+				}, addRequestRetriever(fm))
 			},
 			expectedReqURL:             "https://s3-outposts.us-west-2.amazonaws.com/v20180820/bucket/mockBucket",
 			expectedSigningName:        "s3-outposts",
@@ -607,15 +582,7 @@ func TestCustomEndpoint_SpecialOperations(t *testing.T) {
 				return svc.ListRegionalBuckets(ctx, &s3control.ListRegionalBucketsInput{
 					AccountId: aws.String("123456789012"),
 					OutpostId: aws.String("op-01234567890123456"),
-				}, func(options *s3control.Options) {
-					// append request retriever middleware for request inspection
-					options.APIOptions = append(options.APIOptions,
-						func(stack *middleware.Stack) error {
-							// adds AFTER operation serializer middleware
-							stack.Serialize.Insert(fm, "OperationSerializer", middleware.After)
-							return nil
-						})
-				})
+				}, addRequestRetriever(fm))
 			},
 			expectedReqURL:             "https://s3-outposts.us-west-2.amazonaws.com/v20180820/bucket",
 			expectedSigningName:        "s3-outposts",
@@ -634,15 +601,7 @@ func TestCustomEndpoint_SpecialOperations(t *testing.T) {
 				return svc.ListRegionalBuckets(ctx, &s3control.ListRegionalBucketsInput{
 					AccountId: aws.String("123456789012"),
 					OutpostId: aws.String("op-01234567890123456"),
-				}, func(options *s3control.Options) {
-					// append request retriever middleware for request inspection
-					options.APIOptions = append(options.APIOptions,
-						func(stack *middleware.Stack) error {
-							// adds AFTER operation serializer middleware
-							stack.Serialize.Insert(fm, "OperationSerializer", middleware.After)
-							return nil
-						})
-				})
+				}, addRequestRetriever(fm))
 			},
 			expectedReqURL:             "https://s3-outposts-fips.us-west-2.amazonaws.com/v20180820/bucket",
 			expectedSigningName:        "s3-outposts",
@@ -658,15 +617,7 @@ func TestCustomEndpoint_SpecialOperations(t *testing.T) {
 				return svc.ListRegionalBuckets(ctx, &s3control.ListRegionalBucketsInput{
 					AccountId: aws.String("123456789012"),
 					OutpostId: aws.String("op-01234567890123456"),
-				}, func(options *s3control.Options) {
-					// append request retriever middleware for request inspection
-					options.APIOptions = append(options.APIOptions,
-						func(stack *middleware.Stack) error {
-							// adds AFTER operation serializer middleware
-							stack.Serialize.Insert(fm, "OperationSerializer", middleware.After)
-							return nil
-						})
-				})
+				}, addRequestRetriever(fm))
 			},
 			expectedReqURL:             "https://s3-outposts-fips.us-west-2.amazonaws.com/v20180820/bucket",
 			expectedSigningName:        "s3-outposts",
@@ -685,15 +636,7 @@ func TestCustomEndpoint_SpecialOperations(t *testing.T) {
 				return svc.CreateBucket(ctx, &s3control.CreateBucketInput{
 					Bucket:    aws.String("mockBucket"),
 					OutpostId: aws.String("op-01234567890123456"),
-				}, func(options *s3control.Options) {
-					// append request retriever middleware for request inspection
-					options.APIOptions = append(options.APIOptions,
-						func(stack *middleware.Stack) error {
-							// adds AFTER operation serializer middleware
-							stack.Serialize.Insert(fm, "OperationSerializer", middleware.After)
-							return nil
-						})
-				})
+				}, addRequestRetriever(fm))
 			},
 			expectedReqURL:             "https://s3-outposts-fips.us-gov-west-1.amazonaws.com/v20180820/bucket/mockBucket",
 			expectedSigningName:        "s3-outposts",
@@ -709,15 +652,7 @@ func TestCustomEndpoint_SpecialOperations(t *testing.T) {
 				return svc.CreateBucket(ctx, &s3control.CreateBucketInput{
 					Bucket:    aws.String("mockBucket"),
 					OutpostId: aws.String("op-01234567890123456"),
-				}, func(options *s3control.Options) {
-					// append request retriever middleware for request inspection
-					options.APIOptions = append(options.APIOptions,
-						func(stack *middleware.Stack) error {
-							// adds AFTER operation serializer middleware
-							stack.Serialize.Insert(fm, "OperationSerializer", middleware.After)
-							return nil
-						})
-				})
+				}, addRequestRetriever(fm))
 			},
 			expectedReqURL:             "https://s3-outposts-fips.us-gov-west-1.amazonaws.com/v20180820/bucket/mockBucket",
 			expectedSigningName:        "s3-outposts",
@@ -734,17 +669,9 @@ func TestCustomEndpoint_SpecialOperations(t *testing.T) {
 					AccountId: aws.String("123456789012"),
 					Bucket:    aws.String("arn:aws:s3:us-west-2:123456789012:bucket:mockBucket"),
 					Name:      aws.String("mockName"),
-				}, func(options *s3control.Options) {
-					// append request retriever middleware for request inspection
-					options.APIOptions = append(options.APIOptions,
-						func(stack *middleware.Stack) error {
-							// adds AFTER operation serializer middleware
-							stack.Serialize.Insert(fm, "OperationSerializer", middleware.After)
-							return nil
-						})
-				})
+				}, addRequestRetriever(fm))
 			},
-			expectedErr: "invalid Amazon s3 ARN, unknown resource type",
+			expectedErr: "operation error S3 Control: CreateAccessPoint, invalid Amazon s3 ARN, unknown resource type",
 		},
 		"CreateAccessPoint outpost bucket arn": {
 			options: s3control.Options{
@@ -755,15 +682,7 @@ func TestCustomEndpoint_SpecialOperations(t *testing.T) {
 					AccountId: aws.String("123456789012"),
 					Bucket:    aws.String("arn:aws:s3-outposts:us-west-2:123456789012:outpost:op-01234567890123456:bucket:mockBucket"),
 					Name:      aws.String("mockName"),
-				}, func(options *s3control.Options) {
-					// append request retriever middleware for request inspection
-					options.APIOptions = append(options.APIOptions,
-						func(stack *middleware.Stack) error {
-							// adds AFTER operation serializer middleware
-							stack.Serialize.Insert(fm, "OperationSerializer", middleware.After)
-							return nil
-						})
-				})
+				}, addRequestRetriever(fm))
 			},
 			expectedReqURL:             "https://s3-outposts.us-west-2.amazonaws.com/v20180820/accesspoint/mockName",
 			expectedSigningName:        "s3-outposts",
@@ -782,15 +701,7 @@ func TestCustomEndpoint_SpecialOperations(t *testing.T) {
 					AccountId: aws.String("123456789012"),
 					Bucket:    aws.String("arn:aws:s3-outposts:us-west-2:123456789012:outpost:op-01234567890123456:bucket:mockBucket"),
 					Name:      aws.String("mockName"),
-				}, func(options *s3control.Options) {
-					// append request retriever middleware for request inspection
-					options.APIOptions = append(options.APIOptions,
-						func(stack *middleware.Stack) error {
-							// adds AFTER operation serializer middleware
-							stack.Serialize.Insert(fm, "OperationSerializer", middleware.After)
-							return nil
-						})
-				})
+				}, addRequestRetriever(fm))
 			},
 			expectedReqURL:             "https://s3-outposts-fips.us-west-2.amazonaws.com/v20180820/accesspoint/mockName",
 			expectedSigningName:        "s3-outposts",
@@ -806,15 +717,7 @@ func TestCustomEndpoint_SpecialOperations(t *testing.T) {
 					AccountId: aws.String("123456789012"),
 					Bucket:    aws.String("arn:aws:s3-outposts:us-west-2:123456789012:outpost:op-01234567890123456:bucket:mockBucket"),
 					Name:      aws.String("mockName"),
-				}, func(options *s3control.Options) {
-					// append request retriever middleware for request inspection
-					options.APIOptions = append(options.APIOptions,
-						func(stack *middleware.Stack) error {
-							// adds AFTER operation serializer middleware
-							stack.Serialize.Insert(fm, "OperationSerializer", middleware.After)
-							return nil
-						})
-				})
+				}, addRequestRetriever(fm))
 			},
 			expectedReqURL:             "https://s3-outposts-fips.us-west-2.amazonaws.com/v20180820/accesspoint/mockName",
 			expectedSigningName:        "s3-outposts",
@@ -1046,15 +949,7 @@ func TestInputIsNotModified(t *testing.T) {
 	fm := requestRetrieverMiddleware{}
 	svc := s3control.New(opts)
 	params := s3control.DeleteBucketInput{Bucket: ptr.String(inputBucket)}
-	_, err := svc.DeleteBucket(ctx, &params, func(options *s3control.Options) {
-		// append request retriever middleware for request inspection
-		options.APIOptions = append(options.APIOptions,
-			func(stack *middleware.Stack) error {
-				// adds AFTER operation serializer middleware
-				stack.Serialize.Insert(&fm, "OperationSerializer", middleware.After)
-				return nil
-			})
-	})
+	_, err := svc.DeleteBucket(ctx, &params, addRequestRetriever(&fm))
 
 	if err != nil {
 		t.Fatalf("expect no error, got %v", err.Error())
@@ -1162,10 +1057,10 @@ func TestUseDualStackClientBehavior(t *testing.T) {
 
 func (*requestRetrieverMiddleware) ID() string { return "S3:requestRetrieverMiddleware" }
 
-func (rm *requestRetrieverMiddleware) HandleSerialize(
-	ctx context.Context, in middleware.SerializeInput, next middleware.SerializeHandler,
+func (rm *requestRetrieverMiddleware) HandleFinalize(
+	ctx context.Context, in middleware.FinalizeInput, next middleware.FinalizeHandler,
 ) (
-	out middleware.SerializeOutput, metadata middleware.Metadata, err error,
+	out middleware.FinalizeOutput, metadata middleware.Metadata, err error,
 ) {
 	req, ok := in.Request.(*smithyhttp.Request)
 	if !ok {
@@ -1173,20 +1068,19 @@ func (rm *requestRetrieverMiddleware) HandleSerialize(
 	}
 	rm.request = req
 
-	rm.signingName = awsmiddleware.GetSigningName(ctx)
-	rm.signingRegion = awsmiddleware.GetSigningRegion(ctx)
+	parsed := awstesting.ParseSigV4Signature(req.Header)
+	rm.signingName = parsed.SigningName
+	rm.signingRegion = parsed.SigningRegion
 
-	return next.HandleSerialize(ctx, in)
+	return next.HandleFinalize(ctx, in)
 }
 
-var addRequestRetriever = func(fm *requestRetrieverMiddleware) func(options *s3control.Options) {
+func addRequestRetriever(fm *requestRetrieverMiddleware) func(options *s3control.Options) {
 	return func(options *s3control.Options) {
-		// append request retriever middleware for request inspection
 		options.APIOptions = append(options.APIOptions,
 			func(stack *middleware.Stack) error {
-				// adds AFTER operation serializer middleware
-				stack.Serialize.Insert(fm, "OperationSerializer", middleware.After)
-				return nil
-			})
+				return stack.Finalize.Insert(fm, "Signing", middleware.After)
+			},
+		)
 	}
 }
